@@ -15,6 +15,11 @@
 
 #define BUFFER_SIZE (AVCODEC_MAX_AUDIO_FRAME_SIZE * 2)
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 94, 1)
+#define AV_SAMPLE_FMT_S16 SAMPLE_FMT_S16
+#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
+#endif
+
 int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int16_t *buffer2, const char *file_name, int max_length, int *duration)
 {
 	int i, ok = 0, remaining, length, consumed, buffer_size, codec_ctx_opened = 0;
@@ -28,7 +33,11 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int
 #endif
 	int16_t *buffer;
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 2, 0)
 	if (av_open_input_file(&format_ctx, file_name, NULL, 0, NULL) != 0) {
+#else
+	if (avformat_open_input(&format_ctx, file_name, NULL, NULL) != 0) {
+#endif
 		fprintf(stderr, "ERROR: couldn't open the file\n");
 		goto done;
 	}
@@ -40,11 +49,7 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int
 
 	for (i = 0; i < format_ctx->nb_streams; i++) {
 		codec_ctx = format_ctx->streams[i]->codec;
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 64, 0)
-		if (codec_ctx && codec_ctx->codec_type == CODEC_TYPE_AUDIO) {
-#else
 		if (codec_ctx && codec_ctx->codec_type == AVMEDIA_TYPE_AUDIO) {
-#endif
 			stream = format_ctx->streams[i];
 			break;
 		}
@@ -71,9 +76,9 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int
 		goto done;
 	}
 
-	if (codec_ctx->sample_fmt != SAMPLE_FMT_S16) {
+	if (codec_ctx->sample_fmt != AV_SAMPLE_FMT_S16) {
 #ifdef HAVE_AV_AUDIO_CONVERT
-		convert_ctx = av_audio_convert_alloc(SAMPLE_FMT_S16, codec_ctx->channels,
+		convert_ctx = av_audio_convert_alloc(AV_SAMPLE_FMT_S16, codec_ctx->channels,
 		                                     codec_ctx->sample_fmt, codec_ctx->channels, NULL, 0);
 		if (!convert_ctx) {
 			fprintf(stderr, "ERROR: couldn't create sample format converter\n");
@@ -103,7 +108,7 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int
 
 		while (packet_temp.size > 0) {
 			buffer_size = BUFFER_SIZE;
-#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 25, 0)
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 23, 0)
 			consumed = avcodec_decode_audio2(codec_ctx,
 				buffer1, &buffer_size, packet_temp.data, packet_temp.size);
 #else
@@ -133,7 +138,11 @@ int decode_audio_file(ChromaprintContext *chromaprint_ctx, int16_t *buffer1, int
 			if (convert_ctx) {
 				const void *ibuf[6] = { buffer1 };
 				void *obuf[6] = { buffer2 };
+#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(51, 8, 0)
 				int istride[6] = { av_get_bits_per_sample_format(codec_ctx->sample_fmt) / 8 };
+#else
+				int istride[6] = { av_get_bytes_per_sample(codec_ctx->sample_fmt) };
+#endif
 				int ostride[6] = { 2 };
 				int len = buffer_size / istride[0];
 				if (av_audio_convert(convert_ctx, obuf, ostride, ibuf, istride, len) < 0) {
@@ -198,6 +207,7 @@ int fpcalc_main(int argc, char **argv)
 	int32_t *raw_fingerprint;
 	char *file_name, *fingerprint, **file_names;
 	ChromaprintContext *chromaprint_ctx;
+	int algo = CHROMAPRINT_ALGORITHM_DEFAULT;
 
 	file_names = malloc(argc * sizeof(char *));
 	for (i = 1; i < argc; i++) {
@@ -205,9 +215,26 @@ int fpcalc_main(int argc, char **argv)
 		if (!strcmp(arg, "-length") && i + 1 < argc) {
 			max_length = atoi(argv[++i]);
 		}
+		else if (!strcmp(arg, "-version") || !strcmp(arg, "-v")) {
+			printf("fpcalc version %s\n", chromaprint_get_version());
+			return 0;
+		}
 		else if (!strcmp(arg, "-raw")) {
 			raw = 1;
 		}
+		else if (!strcmp(arg, "-algo") && i + 1 < argc) {
+			const char *v = argv[++i];
+			if (!strcmp(v, "test1")) { algo = CHROMAPRINT_ALGORITHM_TEST1; }
+			else if (!strcmp(v, "test2")) { algo = CHROMAPRINT_ALGORITHM_TEST2; }
+			else if (!strcmp(v, "test3")) { algo = CHROMAPRINT_ALGORITHM_TEST3; }
+			else if (!strcmp(v, "test4")) { algo = CHROMAPRINT_ALGORITHM_TEST4; }
+			else {
+				fprintf(stderr, "WARNING: unknown algorithm, using the default\n");
+			}
+		}
+		else if (!strcmp(arg, "-set") && i + 1 < argc) {
+			i += 1;
+		}
 		else {
 			file_names[num_file_names++] = argv[i];
 		}
@@ -216,8 +243,10 @@ int fpcalc_main(int argc, char **argv)
 	if (!num_file_names) {
 		printf("usage: %s [OPTIONS] FILE...\n\n", argv[0]);
 		printf("Options:\n");
+		printf("  -version      print version information\n");
 		printf("  -length SECS  length of the audio data used for fingerprint calculation (default 120)\n");
 		printf("  -raw          output the raw uncompressed fingerprint\n");
+		printf("  -algo NAME    version of the fingerprint algorithm\n");
 		return 2;
 	}
 
@@ -226,7 +255,19 @@ int fpcalc_main(int argc, char **argv)
 
 	buffer1 = av_malloc(BUFFER_SIZE + 16);
 	buffer2 = av_malloc(BUFFER_SIZE + 16);
-	chromaprint_ctx = chromaprint_new(CHROMAPRINT_ALGORITHM_DEFAULT);
+	chromaprint_ctx = chromaprint_new(algo);
+
+	for (i = 1; i < argc; i++) {
+		char *arg = argv[i];
+		if (!strcmp(arg, "-set") && i + 1 < argc) {
+			char *name = argv[++i];
+			char *value = strchr(name, '=');
+			if (value) {
+				*value++ = '\0';
+				chromaprint_set_option(chromaprint_ctx, name, atoi(value));
+			}
+		}
+	}
 
 	for (i = 0; i < num_file_names; i++) {
 		file_name = file_names[i];
@@ -246,8 +287,9 @@ int fpcalc_main(int argc, char **argv)
 			}
 			printf("FINGERPRINT=");
 			for (j = 0; j < raw_fingerprint_size; j++) {
-				printf("%d%s", raw_fingerprint[j], j + 1 < raw_fingerprint_size ? "," : "\n");
+				printf("%d%s", raw_fingerprint[j], j + 1 < raw_fingerprint_size ? "," : "");
 			}
+			printf("\n");
 			chromaprint_dealloc(raw_fingerprint);
 		}
 		else {
